#include "enl.h"

void md(char *s)
{
   if ((!s)||(!*s)) return;
   mkdir(s,S_IRWXU);
}

int exists(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (stat(s,&st)<0) return 0;
   return 1;
}

int isfile(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (stat(s,&st)<0) return 0;
   if (S_ISREG(st.st_mode)) return 1;
   return 0;
}

int isdir(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (stat(s,&st)<0) return 0;
   if (S_ISDIR(st.st_mode)) return 1;
   return 0;
}

char **ls(char *dir, int *num)
{
   int i,dirlen;
   DIR *dirp;
   char **names;
   struct dirent *dp;
   
   if ((!dir)||(!*dir)) return 0;
   dirp=opendir(dir);
   if (!dirp) 
     {
	*num=0;
	return NULL;
     }
   
     /* count # of entries in dir (worst case) */
     for (dirlen=0;(dp = readdir(dirp))!=NULL;dirlen++);
     if (!dirlen) 
     {
	closedir(dirp);
	*num=dirlen;
	return NULL;
     }
    
    
     /* load up the entries, now that we know how many to make */
     names=(char **)Emalloc(dirlen*sizeof(char *));
     if (!names) return NULL;
    
     rewinddir(dirp);
     for (i=0;i<dirlen;) 
     {
	dp=readdir(dirp);
	if (!dp) break;
	names[i]=(char *)Emalloc(strlen(dp->d_name)+1);
	if (!names) return NULL;
	strcpy(names[i], dp->d_name);
	i++;
     }
   
   if (i<dirlen) dirlen=i;     /* dir got shorter... */
   closedir(dirp);
   *num = dirlen;
   return names;
}

void freestrlist(char **l, int num)
{
   if (!l) return;
   while(num--) if (l[num]) Efree(l[num]);
   Efree(l);
}

void rm(char *s)
{
   if ((!s)||(!*s)) return;
   unlink(s);
}

void mv(char *s, char *ss)
{
   if ((!s)||(!ss)||(!*s)||(!*ss)) return;
   rename(s,ss);
}

void cp(char *s, char *ss)
{
   int i;
   FILE *f,*ff;
   unsigned char buf[1];
   
   if ((!s)||(!ss)||(!*s)||(!*ss)) return;
   if (!exists(s)) return;
   i=filesize(s);
   f=fopen(s,"r");
   if (!f) return;
   ff=fopen(ss,"w");
   if (!ff)
     {
	fclose(f);
	return;
     }
   while (fread(buf,1,1,f)) fwrite(buf,1,1,ff);
   fclose(f);
   fclose(ff);
}

time_t moddate(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (!stat(s,&st)<0) return 0;
   if (st.st_mtime>st.st_ctime) return st.st_mtime;
   else return st.st_ctime;
}

int filesize(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (stat(s,&st)<0) return 0;
   return (int)st.st_size;
}

void cd(char *s)
{
   if ((!s)||(!*s)) return;
   chdir(s);
}

char *cwd(void)
{
   return getcwd(NULL,-1);
}

int permissions(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (!stat(s,&st)<0) return 0;
   return st.st_mode;
}

int owner(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (!stat(s,&st)<0) return 0;
   return st.st_uid;
}

int group(char *s)
{
   struct stat st;
   
   if ((!s)||(!*s)) return 0;
   if (!stat(s,&st)<0) return 0;
   return st.st_gid;
}

char *username(int uid)
{
   FILE *f;
   char s[1024],user[64],home[512],ids[32];
   int id,i,j;
   
   f=fopen("/etc/passwd","r");
   while(fgets(s,1024,f))
     {
	i=0;j=0;
	while (s[i])
	  {
	     user[j]=s[i];
	     if (s[i]==':') 
	       {
		  user[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while (s[i])
	  {
	     ids[j]=s[i];
	     if (s[i]==':') 
	       {
		  ids[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	id=atoi(ids);
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while (s[i])
	  {
	     home[j]=s[i];
	     if (s[i]==':') 
	       {
		  home[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	if (id==uid)
	  {
	     fclose(f);
	     return duplicate(user);
	  }
     }
   return NULL;
}

char *homedir(int uid)
{
   FILE *f;
   char s[1024],user[64],home[512],ids[32];
   int id,i,j;
   
   f=fopen("/etc/passwd","r");
   while(fgets(s,1024,f))
     {
	i=0;j=0;
	while (s[i])
	  {
	     user[j]=s[i];
	     if (s[i]==':') 
	       {
		  user[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while (s[i])
	  {
	     ids[j]=s[i];
	     if (s[i]==':') 
	       {
		  ids[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	id=atoi(ids);
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while (s[i])
	  {
	     home[j]=s[i];
	     if (s[i]==':') 
	       {
		  home[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	if (id==uid)
	  {
	     fclose(f);
	     return duplicate(home);
	  }
     }
   return NULL;
}

char *usershell(int uid)
{
   FILE *f;
   char s[1024],user[64],home[512],ids[32],shell[256];
   int id,i,j;
   
   f=fopen("/etc/passwd","r");
   while(fgets(s,1024,f))
     {
	i=0;j=0;
	while (s[i])
	  {
	     user[j]=s[i];
	     if (s[i]==':') 
	       {
		  user[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while (s[i])
	  {
	     ids[j]=s[i];
	     if (s[i]==':') 
	       {
		  ids[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	id=atoi(ids);
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while ((s[i]!=':')&&(s[i])) i++;
	if (s[i]==':') i++;
	while (s[i])
	  {
	     home[j]=s[i];
	     if (s[i]==':') 
	       {
		  home[j]=0;
		  j=0;i++;
		  break;
	       }
	     i++;j++;
	  }
	while ((s[i])&&(s[i]!=' ')&&(s[i]!='\n')&&(s[i]!='\t'))
	  {
	     shell[j]=s[i];
	     i++;j++;
	  }
	shell[j]=0;
	if (id==uid)
	  {
	     fclose(f);
	     return duplicate(shell);
	  }
     }
   return NULL;
}

char *duplicate(char *s)
{
   char *ss;
   int sz;
   
   if (!s) return NULL;
   sz=strlen(s);
   ss=Emalloc(sz+1);
   strncpy(ss,s,sz+1);
   return ss;
}

char *atword(char *s, int num)
{
   int cnt,i;
   
   if (!s) return NULL;
   cnt=0;i=0;
   
   while (s[i])
     {
	if ((s[i]!=' ')&&(s[i]!='\t'))
	  {
	     if (i==0) cnt++;
	     else if ((s[i-1]==' ')||(s[i-1]=='\t')) cnt++;
	     if (cnt==num) return &s[i];
	  }
	i++;
     }
   return NULL;
}

char *atchar(char *s, char c)
{
   int i;

   if (!s) return NULL;
   i=0;
   while(s[i]!=0)
     {
	if (s[i]==c) return &s[i];
	i++;
     }
   return NULL;
}

void word(char *s, int num, char *wd)
{
   int cnt,i;
   char *start,*finish,*ss,*w;
   
   if (!s) return;
   if (!wd) return;
   if (num<=0) 
     {
	*wd=0;
	return;
     }
   cnt=0;i=0;start=NULL;finish=NULL;ss=NULL;
   w=wd;
   
   while (s[i])
     {
	if ((cnt==num)&&((s[i]==' ')||(s[i]=='\t')))
	  {
	     finish=&s[i];
	     break;
	  }
	if ((s[i]!=' ')&&(s[i]!='\t'))
	  {
	     if (i==0) 
	       {
		  cnt++;
		  if (cnt==num) start=&s[i];
	       }
	     else if ((s[i-1]==' ')||(s[i-1]=='\t')) 
	       {
		  cnt++;
		  if (cnt==num) start=&s[i];
	       }
	  }
	i++;
     }
   if (cnt==num)
     {
	if ((start)&&(finish))
	  {
	     for(ss=start;ss<finish;ss++) *wd++=*ss;
	  }
	else if (start)
	  {
	     for(ss=start;*ss!=0;ss++) *wd++=*ss;
	  }
	*wd=0;
     }
   return;
}

int canread(char *s)
{
   if ((!s)||(!*s)) return 0;
   return access(s,R_OK);
}

int canwrite(char *s)
{
   if ((!s)||(!*s)) return 0;
   return access(s,W_OK);
}

int canexec(char *s)
{
   if ((!s)||(!*s)) return 0;
   return access(s,X_OK);
}

