#include "enl.h"

void SetupVars()
{
   lists.next=NULL;
   mode.mode=MODE_NONE;
}

void SetupSignals()
{
   struct sigaction sa;
   
   sa.sa_handler = HandleSigChild;
   sa.sa_flags = 0;
   sigemptyset (&sa.sa_mask);
   sigaction(SIGCHLD, &sa, (struct sigaction *)0);
}

void SetupX()
{
   int test_event_base,test_error_base,test_v1,test_v2;
   int shape_event_base,shape_error_base;
   Root *root;
   
   /* Open a connection to the diplay nominted by the DISPLAY variable */
   disp=XOpenDisplay(NULL);
   /* if cannot connect to display */
   if (!disp)
     {
	Alert("Enlightenment cannot connect to the display nominated by\n"
	      "your shell's DISPLAY environment variable. You may set this\n"
	      "variable to indicate which display name Enlightenment is to\n"
	      "connect to. It may be that you do not have an Xserver already\n"
	      "running to serve that Display connection, or that you do not\n"
	      "have permission to connect to that display. Please make sure\n"
	      "all is correct before trying again. Run an Xserver by running\n"
	      "xdm or startx first, or contact your local system\n"
	      "administrator, or Xserver vendor, or read the X, xdm and\n"
	      "startx manual pages before proceeding.\n");
	EExit(1);
     }
   /* set up an error handler for then E would normally have fatal X errors */
   XSetErrorHandler((XErrorHandler)HandleXError);
   /* set up a handler for when the X Connection goes down */
   XSetIOErrorHandler((XIOErrorHandler)HandleXIOError);
   /* Check for the Shape Extension */
   if (!XShapeQueryExtension(disp,&shape_event_base,&shape_error_base))
     {
	Alert("FATAL ERROR:\n"
	      "This Xserver does not support the Shape extension.\n"
	      "This is required for Enlightenment to run.\n"
	      "Exiting.\n");
	EExit(1);
     }
   /* check for the XTEST extension */
   if (XTestQueryExtension(disp,&test_event_base,&test_error_base,&test_v1,&test_v2))
     {
	/* make Enlightenment immune to other clients using XGrabServer */
	XTestGrabControl(disp,True);
     }   
   /* record the event base for shape change events */
   event_base_shape=shape_event_base;
   /* initialise imlib */
   imd=ImlibInit(disp);
   /* get info about the root window */
   root=SetupRoot();
   AddItem((void *)root,"ROOT",LIST_TYPE_ROOT);
   /* warn, if necessary about visual problems */
   if (root->visual!=imd->x.visual)
     {
	Alert("WARNING:\n"
	      "The Visual used for the Root Window (Desktop Background) does\n"
	      "not match the visual that Imlib has chosen to use (which\n"
	      "will the the visual with the greatest bit depth / greatest\n"
	      "number of colours). This may result in backgrounds set by\n"
	      "Enlightenment either not working, or looking strange.\n"
	      "You can fix this by running your Xserver with its default\n"
	      "Visual being the one with the highest bit-depth. Please\n"
	      "refer to your Xserver manuals, or contact your system\n"
	      "administrator or Xserver vendor.\n");
     }
   /* warn, if necessary about lack fo multi-head support */
   if (ScreenCount(disp)>1)
     {
	Alert("WARNING:\n"
	      "Your Xserver supports a multi-headed configuration with\n"
	      "multiple screens attached to the one display. Enlightenment\n"
	      "does not currently support multi-headed setups, and as a\n"
	      "result will only manage the first screen (this one).\n");
     }
   /* warn, if necessary about X version problems */
   if (ProtocolVersion(disp)!=11)
     {
	Alert("WARNING:\n"
	      "This is not an X11 Xserver. It infact talks the X%i protocol.\n"
	      "This may mean Enlightenment will either not function, or\n"
	      "function incorrectly. If it is later than X11, then your\n"
	      "server is one the author(s) of Enlightenment neither have\n"
	      "access to, nor have heard of.\n",ProtocolVersion(disp));
     }
   /* sleect all the root window events to start managing */
/*   XSelectInput(disp,root->win,
 *		SubstructureNotifyMask|ButtonPressMask|ButtonReleaseMask|
 *		EnterWindowMask|LeaveWindowMask|ButtonMotionMask|
 *		PropertyChangeMask|SubstructureRedirectMask|KeyPressMask|
 *		KeyReleaseMask|PointerMotionMask);
 */
}
